@@ -16,8 +16,19 @@
 #include <libxml++/nodes/element.h>
 #include <libxml++/dtd.h>
 
+#include <string>
 #include <ostream>
 
+/* std::string or Glib::ustring in function prototypes in libxml++?
+ *
+ * If it's propagated to a libxml2 function that takes a xmlChar*, it's
+ * UTF-8 encoded, and Glib::ustring is the right choice.
+ *
+ * If it's propagated to a libxml2 function that takes a char*, it's not
+ * necessarily UTF-8 encoded, and std::string is usually the right choice.
+ * Most of these strings are filenames or URLs.
+ */
+
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 extern "C" {
   struct _xmlDoc;
@@ -28,64 +39,65 @@ extern "C" {
 namespace xmlpp
 {
 
-typedef enum {
-    XML_INTERNAL_GENERAL_ENTITY = 1,
-    XML_EXTERNAL_GENERAL_PARSED_ENTITY = 2,
-    XML_EXTERNAL_GENERAL_UNPARSED_ENTITY = 3,
-    XML_INTERNAL_PARAMETER_ENTITY = 4,
-    XML_EXTERNAL_PARAMETER_ENTITY = 5,
-    XML_INTERNAL_PREDEFINED_ENTITY = 6
-} XmlEntityType;
-
-class Document;
+// xmlpp::XmlEntityType is similar to xmlEntityType in libxml2.
+/** The valid entity types.
+ */
+enum class XmlEntityType
+{
+  INTERNAL_GENERAL = 1,
+  EXTERNAL_GENERAL_PARSED = 2,
+  EXTERNAL_GENERAL_UNPARSED = 3,
+  INTERNAL_PARAMETER = 4,
+  EXTERNAL_PARAMETER = 5,
+  INTERNAL_PREDEFINED = 6
+};
 
 //TODO: Make Document inherit from Node, when we can break ABI one day?
 //
 //libxml might intend xmlDoc to derive (theoretically) from xmlNode.
-//This is suggested because the xmlNodeSet returned by xmlXPathEval (see the Node::find() implementation) can contain either xmlNode or xmlDocument elements.
+//This is suggested because the xmlNodeSet returned by xmlXPathEval (see the
+//Node::find() implementation) can contain either xmlNode or xmlDocument elements.
+// See https://bugzilla.gnome.org/show_bug.cgi?id=754673#c8 for an explanation
+// why it has not been done in libxml++ 3.0.
 /**
  * Represents an XML document in the DOM model.
  */
-class Document : NonCopyable
+class Document : public NonCopyable
 {
   //Ensure that libxml is properly initialised:
-  class LIBXMLPP_API Init
+  class Init
   {
   public:
     Init();
-
-    //TODO: Remove the virtual when we can break ABI?
-    virtual ~Init();
+    ~Init() noexcept;
   };
 
-  friend LIBXMLPP_API class SaxParser;
+  friend class SaxParser;
 
 public:
   /** Create a new document.
    * @param version XML version.
    * @throws xmlpp::internal_error If memory allocation fails.
    */
-  LIBXMLPP_API
   explicit Document(const Glib::ustring& version = "1.0");
-  
+
   /** Create a new C++ wrapper for an xmlDoc struct.
    * The created xmlpp::Document takes ownership of the xmlDoc.
    * When the Document is deleted, so is the xmlDoc and all its nodes.
    * @param doc A pointer to an xmlDoc struct. Must not be <tt>nullptr</tt>.
+   * @throws xmlpp::internal_error If @a doc is <tt>nullptr</tt>.
    */
-  LIBXMLPP_API explicit Document(_xmlDoc* doc);
-    
-  LIBXMLPP_API ~Document() override;
+  explicit Document(_xmlDoc* doc);
+
+  ~Document() override;
 
   /** @return The encoding used in the source from which the document has been loaded.
    */
-  LIBXMLPP_API
   Glib::ustring get_encoding() const;
 
   /** Get the internal subset of this document.
    * @returns A pointer to the DTD, or <tt>nullptr</tt> if not found.
    */
-  LIBXMLPP_API
   Dtd* get_internal_subset() const;
 
   /** Create the internal subset of this document.
@@ -94,19 +106,21 @@ public:
    * @param external_id The external (PUBLIC) ID, or an empty string.
    * @param system_id The system ID, or an empty string.
    */
-  LIBXMLPP_API
   void set_internal_subset(const Glib::ustring& name,
                            const Glib::ustring& external_id,
                            const Glib::ustring& system_id);
 
-  //TODO: There should be a const and non-const version.
-  //See the patch here: https://bugzilla.gnome.org/show_bug.cgi?id=632522
   /** Return the root node.
    * This function does @b not create a default root node if it doesn't exist.
    * @return A pointer to the root node if it exists, <tt>nullptr</tt> otherwise.
    */
-  LIBXMLPP_API
-  Element* get_root_node() const;
+  Element* get_root_node();
+
+  /** Return the root node.
+   * This function does @b not create a default root node if it doesn't exist.
+   * @return A pointer to the root node if it exists, <tt>nullptr</tt> otherwise.
+   */
+  const Element* get_root_node() const;
 
   /** Create the root element node.
    * If the document already contains a root element node, it is replaced, and
@@ -120,7 +134,6 @@ public:
    * @throws xmlpp::internal_error If memory allocation fails.
    * @throws xmlpp::exception If a new namespace node cannot be created.
    */
-  LIBXMLPP_API
   Element* create_root_node(const Glib::ustring& name,
                             const Glib::ustring& ns_uri = Glib::ustring(),
                             const Glib::ustring& ns_prefix = Glib::ustring() );
@@ -135,7 +148,6 @@ public:
    * @return A pointer to the new root node
    * @throws xmlpp::exception If the node can't be copied.
    */
-  LIBXMLPP_API
   Element* create_root_node_by_import(const Node* node,
 				      bool recursive = true);
 
@@ -144,7 +156,6 @@ public:
    * @returns The new comment node.
    * @throws xmlpp::internal_error
    */
-  LIBXMLPP_API
   CommentNode* add_comment(const Glib::ustring& content);
 
   /** Append a new processing instruction node.
@@ -156,18 +167,15 @@ public:
    * @returns The new processing instruction node.
    * @throws xmlpp::internal_error
    */
-  LIBXMLPP_API
   ProcessingInstructionNode* add_processing_instruction(
     const Glib::ustring& name, const Glib::ustring& content);
 
-  //TODO: Use std::string for filenames.
   /** Write the document to a file.
    * @param filename
    * @param encoding If not provided, UTF-8 is used
    * @throws xmlpp::exception
    */
-  LIBXMLPP_API
-  void write_to_file(const Glib::ustring& filename, const Glib::ustring& encoding = Glib::ustring());
+  void write_to_file(const std::string& filename, const Glib::ustring& encoding = Glib::ustring());
 
   /** Write the document to a file.
    * The output is formatted by inserting whitespaces, which is easier to read for a human,
@@ -176,15 +184,13 @@ public:
    * @param encoding If not provided, UTF-8 is used
    * @throws xmlpp::exception
    */
-  LIBXMLPP_API
-  void write_to_file_formatted(const Glib::ustring& filename, const Glib::ustring& encoding = Glib::ustring());
+  void write_to_file_formatted(const std::string& filename, const Glib::ustring& encoding = Glib::ustring());
 
   /** Write the document to the memory.
    * @param encoding If not provided, UTF-8 is used
    * @returns The written document.
    * @throws xmlpp::exception
    */
-  LIBXMLPP_API
   Glib::ustring write_to_string(const Glib::ustring& encoding = Glib::ustring());
 
   /** Write the document to the memory.
@@ -194,7 +200,6 @@ public:
    * @returns The written document.
    * @throws xmlpp::exception
    */
-  LIBXMLPP_API
   Glib::ustring write_to_string_formatted(const Glib::ustring& encoding = Glib::ustring());
 
   /** Write the document to a std::ostream.
@@ -205,7 +210,6 @@ public:
    * @warning This method is much less efficient than write_to_string if you want to dump the
    * document to a buffer or the standard output. Writing to a fstream is almost as fast as write_to_file
    */
-  LIBXMLPP_API
   void write_to_stream(std::ostream& output, const Glib::ustring& encoding = Glib::ustring());
 
   /** Write the document to a std::ostream.
@@ -217,7 +221,6 @@ public:
    * @throws xmlpp::internal_error
    * @warning See write_to_stream
    */
-  LIBXMLPP_API
   void write_to_stream_formatted(std::ostream & output, const Glib::ustring& encoding = Glib::ustring());
 
   /** Add an Entity declaration to the document.
@@ -229,11 +232,8 @@ public:
    * is the replacement value.
    * @throws xmlpp::internal_error
    */
-  LIBXMLPP_API
-  virtual void set_entity_declaration(const Glib::ustring& name, 
-                                      XmlEntityType type,
-                                      const Glib::ustring& publicId,
-                                      const Glib::ustring& systemId,
+  virtual void set_entity_declaration(const Glib::ustring& name, XmlEntityType type,
+                                      const Glib::ustring& publicId, const Glib::ustring& systemId,
                                       const Glib::ustring& content);
 
   //TODO: When we can break ABI, remove the process_xinclude() with one parameter,
@@ -268,7 +268,7 @@ public:
    * Parser::set_parser_options() and DomParser::set_xinclude_options() do not
    * affect %Document::process_xinclude().
    *
-   * @newin{2,42}
+   * @newin{3,2}
    *
    * @param generate_xinclude_nodes Generate XIncludeStart and XIncludeEnd nodes.
    * @param fixup_base_uris Add or replace xml:base attributes in included element
@@ -276,14 +276,13 @@ public:
    * @returns The number of substitutions.
    * @throws xmlpp::exception
    */
-  LIBXMLPP_API
   int process_xinclude(bool generate_xinclude_nodes, bool fixup_base_uris);
 
   ///Access the underlying libxml implementation.
-  LIBXMLPP_API _xmlDoc* cobj();
+  _xmlDoc* cobj() noexcept;
 
   ///Access the underlying libxml implementation.
-  LIBXMLPP_API const _xmlDoc* cobj() const;
+  const _xmlDoc* cobj() const noexcept;
 
 protected:
   /** Retrieve an Entity.
@@ -291,18 +290,12 @@ protected:
    * @param name The name of the entity to get.
    * @returns A pointer to the libxml2 entity structure, or <tt>nullptr</tt> if not found.
    */
-  LIBXMLPP_API
   _xmlEntity* get_entity(const Glib::ustring& name);
 
 private:
-  //TODO: Remove virtuals when we can break ABI.
-
-  LIBXMLPP_API
-  virtual void do_write_to_file(const Glib::ustring& filename, const Glib::ustring& encoding, bool format);
-  LIBXMLPP_API
-  virtual Glib::ustring do_write_to_string(const Glib::ustring& encoding, bool format);
-  LIBXMLPP_API
-  virtual void do_write_to_stream(std::ostream& output, const Glib::ustring& encoding, bool format);
+  void do_write_to_file(const std::string& filename, const Glib::ustring& encoding, bool format);
+  Glib::ustring do_write_to_string(const Glib::ustring& encoding, bool format);
+  void do_write_to_stream(std::ostream& output, const Glib::ustring& encoding, bool format);
 
   static Init init_;
 
