@@ -1,5 +1,3 @@
-// -*- C++ -*-
-
 /* internal_error.h
  *
  * Copyright (C) 2002 The libxml++ development team
@@ -24,19 +22,16 @@
 
 #include <libxml++/exceptions/exception.h>
 
-#include <exception>
-#include <glibmm/ustring.h>
-
 namespace xmlpp {
-  
-class LIBXMLPP_API internal_error: public exception
+
+class internal_error : public exception
 {
 public:
   explicit internal_error(const Glib::ustring& message);
   ~internal_error() noexcept override;
 
-  void Raise() const override;
-  exception * Clone() const override;
+  void raise() const override;
+  exception* clone() const override;
 };
 
 } // namespace xmlpp
