@@ -38,12 +38,11 @@ namespace xmlpp
 class wrapped_exception : public exception
 {
 public:
-  LIBXMLPP_API
   explicit wrapped_exception(std::exception_ptr exception_ptr);
-  LIBXMLPP_API ~wrapped_exception() noexcept override;
+  ~wrapped_exception() noexcept override;
 
-  LIBXMLPP_API void Raise() const override;
-  LIBXMLPP_API exception* Clone() const override;
+  void raise() const override;
+  exception* clone() const override;
 
 private:
   std::exception_ptr exception_ptr_;
